@@ -1,7 +1,7 @@
 /* magui.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -1399,6 +1399,43 @@ magui_set_zoomfactor_lock_on_off ()
 	
 }
 
+static void
+magui_zoomfactor_lock_toggled (GtkToggleButton *button, gpointer user_data)
+{
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_lock_factor)))
+    {
+	gtk_spin_button_set_value   (GTK_SPIN_BUTTON (sp_factorx), 
+				     magnifier_setting->zoomfactorx);
+
+        gtk_spin_button_set_value   (GTK_SPIN_BUTTON (sp_factory), 
+    				     magnifier_setting->zoomfactorx);
+    }
+}
+
+static void
+magui_zoomfactor_x_value_changed (GtkSpinButton *spin, gpointer user_data)
+{
+    gdouble yval = gtk_spin_button_get_value (GTK_SPIN_BUTTON (sp_factory));
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_lock_factor)))
+    {
+	gdouble xval = gtk_spin_button_get_value (spin);
+	if (xval != yval)
+	    gtk_spin_button_set_value (GTK_SPIN_BUTTON (sp_factory), xval);
+    }    
+}
+
+static void
+magui_zoomfactor_y_value_changed (GtkSpinButton *spin, gpointer user_data)
+{
+    gdouble xval = gtk_spin_button_get_value (GTK_SPIN_BUTTON (sp_factorx));
+    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ck_lock_factor)))
+    {
+	gdouble yval = gtk_spin_button_get_value (spin);
+	if (xval != yval)
+	    gtk_spin_button_set_value (GTK_SPIN_BUTTON (sp_factorx), yval);
+    }
+}
+
 static void
 magui_set_focus_tracking_mode ()
 {
@@ -1574,6 +1611,14 @@ magui_set_handlers_magnification_options(GladeXML *xml)
 
     glade_xml_signal_connect (xml, "on_w_mag_option_remove",		
 			    GTK_SIGNAL_FUNC (magui_magnification_options_remove));
+			    
+    glade_xml_signal_connect (xml, "on_sp_zoomfactor_x_value_changed",
+			      GTK_SIGNAL_FUNC (magui_zoomfactor_x_value_changed));
+			      
+    glade_xml_signal_connect (xml, "on_sp_zoomfactor_y_value_changed",
+			      GTK_SIGNAL_FUNC (magui_zoomfactor_y_value_changed));
+    glade_xml_signal_connect (xml, "on_ck_lock_factor_toggled",
+			      GTK_SIGNAL_FUNC (magui_zoomfactor_lock_toggled));
 }
 
 
@@ -1926,7 +1971,7 @@ magui_magnification_options_value_add_to_widgets (Magnifier *magnifier_setting)
 			    
     magui_set_mouse_tracking (magnifier_setting);
 	
-	magui_set_focus_tracking (magnifier_setting);
+    magui_set_focus_tracking (magnifier_setting);
 
     return TRUE;
 }
