@@ -1841,26 +1841,6 @@ spconf_dictionary_find_word (DictionaryListType *list,
     return elem;
 }
 
-DictionaryListType*
-spconf_dictionary_modify_word (DictionaryListType *list,
-			       const gchar *word,
-			       const gchar *replace)
-{
-    DictionaryListType *elem = NULL;
-    sru_return_val_if_fail (replace, list);
-    sru_return_val_if_fail (word, list);
-    sru_return_val_if_fail (list, list);
-
-    if ((elem = spconf_dictionary_find_word (list, word)) == NULL)
-	return list;
-
-    g_free (elem->data);
-    elem->data = 
-	spconf_dictionary_create_entry (word, replace);
-    
-    return list;
-}
-
 DictionaryListType*
 spconf_dictionary_add_word (DictionaryListType *list,
 			    const gchar *word,
