@@ -1,7 +1,7 @@
 /* SRLow.c
  *
- * Copyright 2001, 2002 Sun Microsystems, Inc.,
- * Copyright 2001, 2002 BAUM Retec, A.G.
+ * Copyright 2001 - 2005 Sun Microsystems, Inc.,
+ * Copyright 2001 - 2005 BAUM Retec, A.G.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,7 +27,9 @@
 #include "string.h"
 #include  "gdk/gdk.h"
 
+#define SRL_MAX_CHILDREN_CNT		1000
 #define	SRL_MAX_CLIENTS			1
+#define SRL_MAX_CHILDREN_CNT 		200	/* depth of child searches */
 #define	SRL_LOG
 /* #define SRL_TIMING */
 
@@ -657,10 +659,15 @@ srl_get_focused_cell (Accessible *acc_table)
 
     if (rows != -1 && cols != -1)
     {
+	/* limit our search to at most SRL_MAX_CHILDREN_CNT cells, in */
+	/* case we get pathological start/end row/col (or have a large */
+	/* screen with a tiny spreadsheet with a gazillion showing cells */
 	gint i, j;
-	for (i = rows; !rv && i <= rowe; i++)
-	    for (j = cols; !rv && j <= cole; j++)
+	gint count = 0;
+	for (i = rows; !rv && i <= rowe && count < SRL_MAX_CHILDREN_CNT; i++)
+	    for (j = cols; !rv && j <= cole && count < SRL_MAX_CHILDREN_CNT; j++)
 	    {
+		count++;
 		Accessible *cell = AccessibleTable_getAccessibleAt (table, i, j);
 		if (cell)
 		{
@@ -680,6 +687,40 @@ srl_get_focused_cell (Accessible *acc_table)
     return rv;
 }
 
+/* function returns the active child in a tree. The search
+   is limited to maximum _limit_ number of children, children's chidren etc */
+static Accessible*
+srl_tree_get_active_item (Accessible *tree,
+			  gint *limit)
+{
+    Accessible *active = NULL;
+    gint cnt, i;
+
+    srl_assert (tree);
+
+    cnt = Accessible_getChildCount (tree);
+    for (i = 0; i < cnt; i++)
+    {
+	Accessible *child;
+	child = Accessible_getChildAtIndex (tree, i);
+	if (!child)
+	    continue;
+	(*limit)--;
+	if (srl_acc_has_state (child, SPI_STATE_ACTIVE))
+	{
+	    active = child;
+	    Accessible_ref (active);
+	}
+	else if (srl_acc_has_state (child, SPI_STATE_EXPANDED))
+	    active = srl_tree_get_active_item (child, limit);
+	Accessible_unref (child);
+	if (active || *limit < 0)
+	    break;
+    }
+    
+    return active;
+}
+
 static gboolean
 srle_acc_has_state (SRLEvent *event,
 		    AccessibleState state)
@@ -764,6 +805,13 @@ srle_set_acc (SRLEvent *event)
 	    AccessibleSelection_unref (selection);
 	}
     }
+    else if (srle_has_type (event, SRL_EVENT_FOCUS) &&
+	     srle_acc_has_role (event, SPI_ROLE_TREE))
+    {
+	gint limit = SRL_MAX_CHILDREN_CNT;
+	event->acc = srl_tree_get_active_item ((Accessible*)srle_get_acc (event), 
+						 &limit);
+    }
     else if (srle_has_type (event, SRL_EVENT_ACTIVE_DESCENDANT_CHANGED))
     {
 	event->acc = AccessibleActiveDescendantChangedEvent_getActiveDescendant (event->acc_ev);
@@ -1547,7 +1595,8 @@ srle_change_type (SRLEvent *event)
 {
     srl_assert (event);
     
-    if (srle_has_type (event, SRL_EVENT_ACTIVE_DESCENDANT_CHANGED))
+    if (srle_has_type (event, SRL_EVENT_ACTIVE_DESCENDANT_CHANGED) && 
+	srle_is_for_focused_ancestor_acc (event))
     {
 	event->type = SRL_EVENT_FOCUS;
     }
